/**
 *
 */

#include "HydroArray.h"
#include <stdio.h> // for printf

#include "../hydroParams.h" // for varNames

/*********************************************************/

/**
 * Create a new HydroArray2d, allocate memory and retrun pointer to it.
 */
HydroArray2d * HydroArray2d_new(int isize, int jsize, int nVar) {

  /* memory allocation for HydroArray2d structure */
  HydroArray2d* result = malloc( sizeof(HydroArray2d) );

  /* initialize Array1d members */
  result->isize   = isize;
  result->jsize   = jsize;
  result->nVar = nVar;
  result->data = (real_t *) malloc (isize * jsize * nVar * sizeof(real_t) );

  // here we should check that memory allocation was OK. TODO.

  /* return structure pointer */
  return result;

} // HydroArray2d_new

/**
 * Free memory for a struct HydroArray2d.
 */
void HydroArray2d_free(HydroArray2d *array) {

  /* free memory of array members */
  free(array->data);

  /* free memory of array itself */
  free(array);

} // HydroArray2d_free

/**
 * Print HydroArray2d data on stdout.
 */
void HydroArray2d_print(const HydroArray2d* array) {
  
  // print some header ?
  printf("-----------------------------------\n");
  printf("dump data for HydroArray2d at address %p\n",array);

  // print data
  for (int ivar=0; ivar<array->nVar; ivar++) {
    printf("ivar=%d (%s)\n",ivar,varNames[ivar]);
    for (int j=0; j<array->jsize; j++) {
      for (int i=0; i<array->isize; i++)
	printf("%f ",array->data[i                               +
				 j    * array->isize             +
				 ivar * array->isize*array->jsize]);
      printf("\n");
    }
  }

  // print footer
  printf("-----------------------------------\n");

} // HydroArray2d_print

/**
 * Get value at location idx.
 */
real_t HydroArray2d_get_idx(const HydroArray2d *array, int idx) {

  return array->data[idx];

} // HydroArray2d_get_idx

/**
 * Set value at location idx.
 */
void HydroArray2d_set_idx(HydroArray2d *array, int idx, real_t value) {

  array->data[idx] = value;

} // HydroArray2d_set_idx

/**
 * Get value at location i,j,ivar.
 */
real_t HydroArray2d_get(const HydroArray2d *array, int i, int j, int ivar) {

  // compute linear index
  int idx = i + j*array->isize + ivar * array->isize*array->jsize;

  // get value
  return array->data[idx];

} // HydroArray2d_get

/**
 * Set value at location i,j,ivar.
 */
void HydroArray2d_set(HydroArray2d *array, int i, int j, int ivar, real_t value) {

  // compute linear index
  int idx = i + j*array->isize + ivar * array->isize*array->jsize;

  // set value
  array->data[idx] = value;

} // HydroArray2d_set

/**
 * Set value at location i,j,ivar.
 */
void HydroArray2d_incr(HydroArray2d *array, int i, int j, int ivar, real_t value) {

  // compute linear index
  int idx = i + j*array->isize + ivar * array->isize*array->jsize;

  // set value
  array->data[idx] += value;

} // HydroArray2d_incr
