/**
 *
 */
#ifndef HYDRO_UTILS_H_
#define HYDRO_UTILS_H_

#include "real_type.h"
#include "hydroParams.h"
#include "array/HydroArray.h"

void swapValues(real_t *a, real_t *b);
void eos(real_t rho, real_t eint, real_t* p, real_t* c);

void computePrimitives(const real_t u[nbVar], real_t* c, real_t q[nbVar]);
void computePrimitives_ij(const HydroArray2d* U, int i, int j, real_t* c, real_t q[nbVar]);

void trace_unsplit_2d(const real_t q[nbVar], 
		      const real_t qNeighbors[2*NDIM][nbVar],
		      real_t c, 
		      real_t dtdx, 
		      real_t dtdy, 
		      real_t qm[NDIM][nbVar], 
		      real_t qp[NDIM][nbVar]);

void trace_unsplit_2d_along_dir(const real_t q[nbVar], 
				const real_t dqX[nbVar],
				const real_t dqy[nbVar],
				real_t dtdx, 
				real_t dtdy, 
				int    faceId,
				real_t qface[nbVar]);

void trace_unsplit_hydro_2d(const real_t q[nbVar],
			    const real_t dq[2][nbVar],
			    real_t dtdx,
			    real_t dtdy,
			    real_t qm[2][nbVar], 
			    real_t qp[2][nbVar]);

void slope_unsplit_hydro_2d(const real_t q[nbVar], 
			    const real_t qPlusX[nbVar], 
			    const real_t qMinusX[nbVar],
			    const real_t qPlusY[nbVar], 
			    const real_t qMinusY[nbVar],
			    real_t dqX[nbVar],
			    real_t dqY[nbVar]);

void cmpflx(const real_t qgdnv[nbVar], 
	    real_t flux[nbVar]);

void riemann_approx(const real_t qleft[nbVar], 
		    const real_t qright[nbVar], 
		    real_t qgdnv[nbVar], 
		    real_t flux[nbVar]);

void riemann_hllc(const real_t qleft[nbVar], 
		  const real_t qright[nbVar], 
		  real_t qgdnv[nbVar], 
		  real_t flux[nbVar]);

#endif // HYDRO_UTILS_H_
